/********************************** (C) COPYRIGHT *******************************
* File Name          : usb30_desc.c
* Author             : WCH
* Version            : V1.0
* Date               : 2020/07/31
* Description 		 : 
* Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
* SPDX-License-Identifier: Apache-2.0
*******************************************************************************/
#include "usb30_desc.h"

/* Device Descriptor */
const UINT8 DeviceDescriptor[] =
{
	0x12,
	0x01,
	0x00,
	0x03,//bcdUSB
	0xef,
	0x02,
	0x01,//Device Class
	0x09,//Max packet 512
	0x86,
	0x1A, //VID
	0x04,
	0x23,//PID
	0x06,
	0x01,//bcdDevice
	0x03,//iManufacturer
	0x01,//iProduct
	0x02,//iSerialNumber
	0x01,//bNumConfigurations
};

/* Configuration Descriptor */
const UINT8 ConfigDescriptor[] =
{
        /* Configuration Descriptor */
        0x09,                                                                       /* bLength */
        0x02,                                                                       /* bDescriptorType */
        0x69,
        0x02,                                                                       /* wTotalLength */
        0x02,                                                                       /* bNumInterfaces */
        0x01,                                                                       /* bConfigurationValue */
        0x00,                                                                       /* iConfiguration */
        0x80,                                                                       /* bmAttributes */
        0xFA,                                                                       /* bMaxPower */

        /* Interface Association Descriptor */
        0x08,                                                                       /* bLength */
        0x0b,                                                                       /* bDescriptorType: INTERFACE ASSOCIATION Descriptor */
        0x00,                                                                       /* bFirstInterface: Interface number of the VideoControl interface */
        0x02,                                                                       /* bInterfaceCount: Number of contiguous Video interfaces */
        0x0e,                                                                       /* bFunctionClass: CC_VIDEO */
        0x03,                                                                       /* bFunctionSubClass: SC_VIDEO_INTERFACE_COLLECTION */
        0x00,                                                                       /* bFunctionProtocol:  */
        0x02,                                                                       /* iFunction */

        /* Standard VC Interface Descriptor */
        0x09,                                                                       /* bLength */
        0x04,                                                                       /* bDescriptorType */
        0x00,                                                                       /* bInterfaceNumber */
        0x00,                                                                       /* bAlternateSetting */
        0x01,                                                                       /* bNumEndpoints */
        0x0e,                                                                       /* bInterfaceClass: CC_VIDEO */
        0x01,                                                                       /* bInterfaceSubClass: SC_VIDEOCONTROL */
        0x00,                                                                       /* bInterfaceProtocol: PC_PROTOCOL_15 */
        0x02,                                                                       /* iInterface */

        /* Class-specific VC Interface Descriptor */
        0x0d,                                                                       /* bLength */
        0x24,                                                                       /* bDescriptorType: CS_INTERFACE */
        0x01,                                                                       /* bDescriptorSubType: VC_HEADER subtype */
        0x00,                                                                       /* bcdUVC */
        0x01,
        0x4D,                                                                       /* wTotalLength: Total size of class-specific descriptors */
        0x00,
        0x80,                                                                       /* dwClockFrequency */
        0xC3,
        0xC9,
        0x01,
        0x01,                                                                       /* bInCollectionNumber of streaming interfaces  */
        0x01,                                                                       /* baInterfaceNr(1) */

        /* Input Terminal Descriptor (Composite) */
        0x12,                                                                       /* bLength: 8 + x */
        0x24,                                                                       /* bDescriptorType: CS_INTERFACE */
        0x02,                                                                       /* bDescriptorSubtype: VC_INPUT_TERMINAL subtype */
        0x01,                                                                       /* bTerminalID: ID of this input terminal */
        0x01,                                                                       /* wTerminalType: ITT_CAMERA type */
        0x02,
        0x00,                                                                       /* bAssocTerminal: ID of the Output Terminal to which this Input Terminal is associated. */
        0x00,                                                                       /* Index of a string descriptor that describes the Camera Terminal. */
        0x00,
        0x00,                                                                       /* The value of Lmin If Optical Zoom is not supported; this field shall be set to 0. */
        0x00,
        0x00,                                                                       /* The value of Lmax If Optical Zoom is not supported; this field shall be set to 0 */
        0x00,
        0x00,                                                                       /* The value of Locular If Optical Zoom is not supported; this field shall be set to 0 */
        0x03,                                                                       /* Size in bytes of the bmControls field: 3 */
        0x00,
        0x00,
        0x00,

        /* Output Terminal Descriptor */
        0x09,                                                                       /* bLength */
        0x24,                                                                       /* bDescriptorType: CS_INTERFACE */
        0x03,                                                                       /* bDescriptorSubtype: VC_OUTPUT_TERMINAL */
        0x02,                                                                       /* bTerminalID: ID of this terminal */
        0x01,                                                                       /* wTerminalType: TT_STREAMING type. This terminal is a USB streaming terminal. */
        0x01,
        0x00,                                                                       /* bAssocTerminal */
        0x04,                                                                       /* bSourceID */
        0x00,                                                                       /* iTerminal */

        /* Extension Unit Descriptor */
        0x0B,
        0x24,
        0x05,
        0x03,
        0x01,
        0x00,
        0x00,
        0x02,
        0x3B,
        0x05,
        0x00,


        /* Extension Unit Descriptor */
        0x1A,
        0x24,
        0x06,
        0x04,
        0x52,
        0xF2,
        0xB8,
        0xAA,
        0xD1,
        0x8E,
        0x72,
        0x49,
        0x8C,
        0xED,
        0x96,
        0xB1,
        0x7F,
        0x04,
        0x40,
        0x8B,
        0x01,
        0x01,
        0x03,
        0x01,
        0x01,
        0x00,

        /* Standard VC Interrupt Endpoint Descriptor */
        0x07,                                                                       /* bLength */
        0x05,                                                                       /* bDescriptorType: ENDPOINT descriptor type */
        0x83,                                                                       /* bEndpointAddress */
        0x03,                                                                       /* bmAttributes */
        0x10,                                                                       /* wMaxPacketSize: 2bytes */
        0x00,
        0x06,                                                                       /* bInterval: For high-speed: 2^(bInterval-1) */

        0x06,0x30,0x00,0x00,0x10,0x00,
        /* Class-specific VC Interrupt Endpoint Descriptor */
        0x05,                                                                       /* bLength */
        0x25,                                                                       /* bDescriptorType: CS_ENDPOINT descriptor type */
        0x03,                                                                       /* bDescriptorSubType: EP_INTERRUPT descriptor type */
        0x40,                                                                       /* wMaxTransferSize: 2bytes */
        0x00,

        /* Standard VS Interface Descriptor */
        0x09,                                                                       /* bLength */
        0x04,                                                                       /* bDescriptorTypeINTERFACE descriptor type  */
        0x01,                                                                       /* bInterfaceNumber */
        0x00,                                                                       /* bAlternateSetting */
        0x00,                                                                       /* bNumEndpoints */
        0x0e,                                                                       /* bInterfaceClassCC_VIDEO  */
        0x02,                                                                       /* bInterfaceSubClassSC_VIDEOSTREAMING  */
        0x00,                                                                       /* bInterfaceProtocol */
        0x00,                                                                       /* iInterface */

        /* VS Interface Input Header Descriptor */
        0x0e,                                                                       /* bLength: 13+(p*n) */
        0x24,                                                                       /* bDescriptorType: CS_INTERFACE descriptor type */
        0x01,                                                                       /* bDescriptorSubtype: VS_INPUT_HEADER descriptor subtype */
        0x01,                                                                       /* bNumFormats: p */
        0x47,                                                                       /* wTotalLength: 2bytes */
        0x01,
        0x81,                                                                       /* bEndpointAddress */
        0x00,                                                                       /* bmInfo */
        0x02,                                                                       /* bTerminalLink */
        0x02,                                                                       /* bStillCaptureMethod */
        0x01,                                                                       /* bTriggerSupport */
        0x00,                                                                       /* bTriggerUsage */
        0x01,                                                                       /* bControlSize: n */
        0x00,                                                                       /* bmaControls(1) */


        /* Class-specific VS Format Descriptor--- VS_FORMAT_MJPEG */
        0x0b,                                                                       /* bLength */
        0x24,                                                                       /* bDescriptorTypeCS_INTERFACE */
        0x06,                                                                       /* bDescriptorSubtype VS_FORMAT_MJPEG */
        0x01,                                                                       /* bFormatIndexFirst (and only) format descriptor  */
        0x0a,                                                                       /* bNumFrameDescriptors10 frame descriptor for this format follows */
        0x01,                                                                       /* bmFlagsUses fixed size samples */
        0x01,                                                                       /* bDefaultFrameIndexDefault frame index is 1. */
        0x00,                                                                       /* bAspectRatioX */
        0x00,                                                                       /* bAspectRatioY */
        0x00,                                                                       /* bmInterlaceFlags */
        0x00,                                                                       /* bCopyProtect */

        /* 800*600@30fps */
        0x1e,                                                                       /* bLength: 10+(4*n)-4+m */
        0x24,                                                                       /* bDescriptorType: CS_INTERFACE descriptor type */
        0x07,                                                                       /* bDescriptorSubtype: VS_FRAME_MJPEG */
        0x01,                                                                       /* bFrameIndex: First (and only) frame descriptor */
        0x00,                                                                       /* bmCapabilities */
        0x20,0x03,                                                                  /* Width of frame800 */
        0x58,0x02,                                                                  /* Height of frame 600 */
        0x00,0xA0,0xBB,0x0D,                                                        /* Min bit rate in bits/s */
        0x00,0xA0,0xBB,0x0D,                                                        /* Max bit rate in bits/s 800*600*2*30fps * 8*/
        0x00,0xA6,0x0E,0x00,                                                        /* Maximum video or still frame size, in bytes 800*600*2  */
        0x15,0x16,0x05,0x00,                                                        /* Default frame interval is 30fps(333333ns) */
        0x01,                                                                       /* bFrameIntervalType: Continuous frame interval */
        0x15,0x16,0x05,0x00,                                                        /* Default frame interval is 30fps(333333ns) */
//        0x2a,0x2c,0x0a,0x00,                                                        /* 15fps(666666ns) */


        /* 320*180@30fps */
        0x1e,
        0x24,
        0x07,
        0x02,
        0x00,
        0x40,0x01,
        0xb4,0x00,
        0x00,0xe0,0xa5,0x01,
        0x00,0xe0,0xa5,0x01,
        0x00,0xc2,0x01,0x00,
        0x15,0x16,0x05,0x00,
        0x01,
        0x15,0x16,0x05,0x00,

        /* 320*240@30fps */
        0x1e,
        0x24,
        0x07,
        0x03,
        0x00,
        0x40,0x01,
        0xf0,0x00,
        0x00,0x80,0x32,0x02,
        0x00,0x80,0x32,0x02,
        0x00,0x58,0x02,0x00,
        0x15,0x16,0x05,0x00,
        0x01,
        0x15,0x16,0x05,0x00,

        /* 640*360@30fps */
        0x1e,
        0x24,
        0x07,
        0x04,
        0x00,
        0x80,0x02,
        0x68,0x01,
        0x00,0x80,0x97,0x06,
        0x00,0x80,0x97,0x06,
        0x00,0x08,0x07,0x00,
        0x15,0x16,0x05,0x00,
        0x01,
        0x15,0x16,0x05,0x00,

        /* 640*480@30fps */
        0x1e,
        0x24,
        0x07,
        0x05,
        0x00,
        0x80,0x02,
        0xe0,0x01,
        0x00,0x00,0xca,0x08,
        0x00,0x00,0xca,0x08,
        0x00,0x60,0x09,0x00,
        0x15,0x16,0x05,0x00,
        0x01,
        0x15,0x16,0x05,0x00,

        /* 848*480@30fps */
        0x1e,
        0x24,
        0x07,
        0x06,
        0x00,
        0x50,0x03,
        0xe0,0x01,
        0x00,0x40,0xa5,0x0b,
        0x00,0x40,0xa5,0x0b,
        0x00,0x6c,0x0c,0x00,
        0x15,0x16,0x05,0x00,
        0x01,
        0x15,0x16,0x05,0x00,

        /* 960*540@30fps */
        0x1e,
        0x24,
        0x07,
        0x07,
        0x00,
        0xc0,0x03,
        0x1c,0x02,
        0x00,0xe0,0xd4,0x0e,
        0x00,0xe0,0xd4,0x0e,
        0x00,0xd2,0x0f,0x00,
        0x15,0x16,0x05,0x00,
        0x01,
        0x15,0x16,0x05,0x00,

        /* 1024*768@30fps */
        0x1e,                                                                       /* bLength: 10+(4*n)-4+m */
        0x24,                                                                       /* bDescriptorType: CS_INTERFACE descriptor type */
        0x07,                                                                       /* bDescriptorSubtype: VS_FRAME_MJPEG */
        0x08,                                                                       /* bFrameIndex: First (and only) frame descriptor */
        0x00,                                                                       /* bmCapabilities */
        0x00,0x04,                                                                  /* Width of frame1024 */
        0x00,0x03,                                                                   /* Height of frame 768 */
        0x00,0x00,0x80,0x16,                                                        /* Min bit rate in bits/s */
        0x00,0x00,0x80,0x16,                                                        /* Max bit rate in bits/s 1024*768*2*60fps * 8*/
        0x00,0x00,0x18,0x00,                                                        /* Maximum video or still frame size, in bytes 1024*768*2  */
        0x15,0x16,0x05,0x00,                                                        /* Default frame interval is 30fps(333333ns) */
        0x01,                                                                       /* bFrameIntervalType: Continuous frame interval */
        0x15,0x16,0x05,0x00,

        /* 1280*720@30fps */
        0x1e,
        0x24,
        0x07,
        0x09,
        0x00,
        0x00,0x05,
        0xd0,0x02,
        0x00,0x00,0x5e,0x1a,
        0x00,0x00,0x5e,0x1a,
        0x00,0x20,0x1c,0x00,
        0x15,0x16,0x05,0x00,
        0x01,
        0x15,0x16,0x05,0x00,

        /* 1600*1200@30fps */
        0x1e,
        0x24,
        0x07,
        0x0A,
        0x00,
        0x40,0x06,
        0xb0,0x04,
        0x00,0x80,0xee,0x36,
        0x00,0x80,0xee,0x36,
        0x00,0x98,0x3a,0x00,
        0x15,0x16,0x05,0x00,
        0x01,
        0x15,0x16,0x05,0x00,

        /* Still Image Frame Descriptor */
        0x2e,                                                                       /* bLength: 10+(4*n)-4+m */
        0x24,                                                                       /* bDescriptorType: CS_INTERFACE descriptor type */
        0x03,                                                                       /* bDescriptorSubtype: VS_STILL_IMAGE_FRAME descriptor subtype */
        0x00,                                                                       /* bEndpointAddress: ??? */
        0x0a,                                                                       /* bNumImageSizePatterns: n */
        0x20,0x03,                                                                  /* Width(1):  800 */
        0x58,0x02,                                                                  /* Height(1): 600 */
        0x40,0x01,                                                                  /* Width(2):  320 */
        0xb4,0x00,                                                                  /* Height(2): 180 */
        0x40,0x01,                                                                  /* Width(3):  320 */
        0xf0,0x00,                                                                  /* Height(3): 240 */
        0x80,0x02,                                                                  /* Width(4):  640 */
        0x68,0x01,                                                                  /* Height(4): 360 */
        0x80,0x02,                                                                  /* Width(5):  640 */
        0xe0,0x01,                                                                  /* Height(5): 480 */
        0x50,0x03,                                                                  /* Width(6):  848 */
        0xe0,0x01,                                                                  /* Height(6): 480 */
        0xc0,0x03,                                                                  /* Width(7):  960 */
        0x1c,0x02,                                                                  /* Height(7): 540 */
        0x00,0x04,                                                                  /* Width(8):  1024 */
        0x00,0x03,                                                                  /* Height(8): 768 */
        0x00,0x05,                                                                  /* Width(9):  1280 */
        0x00,0x02,                                                                  /* Height(9): 720 */
        0x40,0x06,                                                                  /* Width(10):  1600 */
        0xb0,0x04,                                                                  /* Height(10): 1200 */
        0x00,                                                                       /* bNumCompressionPtn: m */

        /* Color Matching Descriptor */
        0x06,                                                                       /* bLength */
        0x24,                                                                       /* bDescriptorType: CS_INTERFACE */
        0x0d,                                                                       /* bDescriptorSubtype: VS_COLORFORMAT */
        0x01,                                                                       /* bColorPrimaries: 1: BT.709, sRGB (default) */
        0x01,                                                                       /* bTransferCharacteristics: 1: BT.709 (default) */
        0x04,                                                                       /* bMatrixCoefficients: 4: SMPTE 170M (BT.601, default) */

        /* Asynchronous Isochronous Interface 1 -- configuration value 1 -- endpoint 1 (synchronous) -- endpoint size 524 bytes */
        0x09,0x04,0x01,0x01,0x01,0x0e,0x02,0x00,0x00,
        0x07,0x05,0x81,0x05,0x0c,0x02,0x01,
        0x06,0x30,0x00,0x00,0x00,0x04,

        /* Asynchronous Isochronous Interface 1 -- configuration value 2 -- endpoint 1 (synchronous) -- endpoint size 780 bytes */
        0x09,0x04,0x01,0x02,0x01,0x0e,0x02,0x00,0x00,
        0x07,0x05,0x81,0x05,0x0c,0x03,0x01,
        0x06,0x30,0x00,0x00,0x00,0x04,


        /* Asynchronous Isochronous Interface 1 -- configuration value 3 -- endpoint 1 (synchronous) -- endpoint size 1020 bytes */
        0x09,0x04,0x01,0x03,0x01,0x0e,0x02,0x00,0x00,
        0x07,0x05,0x81,0x05,0xfc,0x03,0x01,
        0x06,0x30,0x00,0x00,0x00,0x04,


        /* Asynchronous Isochronous Interface 1 -- configuration value 4 -- endpoint 1 (synchronous) -- endpoint size 1024 bytes */
        0x09,0x04,0x01,0x04,0x01,0x0e,0x02,0x00,0x00,
        0x07,0x05,0x81,0x05,0x00,0x04,0x01,
        0x06,0x30,0x00,0x00,0x00,0x04,


        /* Asynchronous Isochronous Interface 1 -- configuration value 5 -- endpoint 1 (synchronous) -- endpoint size 5116 bytes */
        0x09,0x04,0x01,0x05,0x01,0x0e,0x02,0x00,0x00,
        0x07,0x05,0x81,0x05,0x00,0x04,0x01,
        0x06,0x30,0x00,0x00,0x00,0x04,

};

/* Language Descriptor */
const UINT8 StringLangID[SIZE_STRING_LANGID] =
{
   SIZE_STRING_LANGID,	// this descriptor length
	0x03,	            // descriptor type
	0x09,	            // Language ID 0 low byte
	0x04	            // Language ID 0 high byte
};

/* Manufacturer Descriptor */
const UINT8 StringVendor[SIZE_STRING_VENDOR] =
{
	SIZE_STRING_VENDOR,
	0x03,
	'W', 0, 'C', 0, 'H', 0, 'I', 0, 'C', 0, 'r', 0, 'o', 0, 'e', 0,
	'l', 0, 'e', 0, 'c', 0, 't', 0, 'r', 0, 'o', 0, 'n', 0, 'i', 0,
	'c', 0, 's', 0
};

/* Product Descriptor */
const UINT8 StringProduct[SIZE_STRING_PRODUCT] =
{
	SIZE_STRING_PRODUCT,
	0x03,
	'C', 0, 'H', 0, '5', 0, '6', 0, '9', 0, '_', 0,'C', 0, 'a', 0,
	'm', 0, 'e', 0, 'r', 0, 'a', 0
};

/* Serial Descriptor */
UINT8 StringSerial[SIZE_STRING_SERIAL] =
{
	SIZE_STRING_SERIAL,	// length of this descriptor
	0x03,
	'0',
	0x00,
	'1',
	0x00,
	'2',
	0x00,
	'3',
	0x00,
	'4',
	0x00,
	'5',
	0x00,
	'6',
	0x00,
	'7',
	0x00,
	'8',
	0x00,
	'9',
	0x00,
};

const UINT8 OSStringDescriptor[SIZE_STRING_OS] =
{
	SIZE_STRING_OS,	// length of this descriptor
	0x03,
	'M',
	0x00,
	'S',
	0x00,
	'F',
	0x00,
	'T',
	0x00,
	'1',
	0x00,
	'0',
	0x00,
	'0',
	0x00,
	0x01,
	0x00
};

const UINT8 BOSDescriptor[SIZE_BOS_DESC] =
{
	0x05,   // length of this descriptor
	0x0f,   // CONFIGURATION (2)
	0x16,   // total length includes endpoint descriptors (should be 1 more than last address)
	0x00,   // total length high byte
	0x02,   // number of device cap
	0x07,
	0x10,	// DEVICE CAPABILITY type
	0x02,	// USB2.0 EXTENSION
	0x06,
	0x00,
	0x00,
	0x00,
	0x0a,	// length of this descriptor
	0x10,	// DEVICE CAPABILITY type
	0x03,	// superspeed usb device capability
	0x00,	//
	0x0e,	// ss/hs/fs
	0x00,
	0x01,	// the lowest speed is full speed
	0x0a,	// u1 exit latency is 10us
	0xff,	// u1 exit latency is 8us
	0x07
};

const UINT8 MSOS20DescriptorSet[] =
{
    // Microsoft OS 2.0 Descriptor Set Header
    0x0A, 0x00,             // wLength - 10 bytes
    0x00, 0x00,             // MSOS20_SET_HEADER_DESCRIPTOR
	0x00, 0x00, 0x03, 0x06, // dwWindowsVersion  0x06030000 for Windows Blue
    0x48, 0x00,             // wTotalLength  72 bytes
    // Microsoft OS 2.0 Registry Value Feature Descriptor
    0x3E, 0x00,             // wLength - 62 bytes
    0x04, 0x00,             // wDescriptorType  4 for Registry Property
    0x04, 0x00,             // wPropertyDataType - 4 for REG_DWORD
    0x30, 0x00,             // wPropertyNameLength  48 bytes
    0x53, 0x00, 0x65, 0x00, // Property Name - SelectiveSuspendEnabled
    0x6C, 0x00, 0x65, 0x00,
    0x63, 0x00, 0x74, 0x00,
    0x69, 0x00, 0x76, 0x00,
    0x65, 0x00, 0x53, 0x00,
    0x75, 0x00, 0x73, 0x00,
    0x70, 0x00, 0x65, 0x00,
    0x6E, 0x00, 0x64, 0x00,
    0x45, 0x00, 0x6E, 0x00,
    0x61, 0x00, 0x62, 0x00,
    0x6C, 0x00, 0x65, 0x00,
    0x64, 0x00, 0x00, 0x00,
    0x04, 0x00,             // wPropertyDataLength 4 bytes
    0x01, 0x00, 0x00, 0x00  // PropertyData - 0x00000001
};

const UINT8 PropertyHeader[] =
{
	0x8e, 0x00, 0x00, 0x00, 0x00, 01, 05, 00, 01, 00,
	0x84, 0x00, 0x00, 0x00,
	0x01, 0x00, 0x00, 0x00,
	0x28, 0x00,
	0x44, 0x00, 0x65, 0x00, 0x76, 0x00, 0x69, 0x00, 0x63, 0x00, 0x65, 0x00, 0x49, 0x00, 0x6e,
	0x00, 0x74, 0x00, 0x65, 0x00, 0x72, 0x00, 0x66, 0x00, 0x61, 0x00, 0x63, 0x00, 0x65, 0x00, 0x47, 0x00, 0x55, 0x00, 0x49, 0x00, 0x44, 0x00, 0x00, 0x00,

	0x4e, 0x00, 0x00, 0x00, //L"{12345678-1234-1234-1234-123456789ABC"
	0x7b, 0x00, 0x31, 0x00, 0x32, 0x00, 0x33, 0x00, 0x34, 0x00, 0x35, 0x00, 0x36, 0x00, 0x37, 0x00, 0x38, 0x00,
	0x2d, 0x00, 0x31, 0x00, 0x32, 0x00, 0x33, 0x00, 0x34, 0x00,
	0x2d ,0x00, 0x31, 0x00, 0x32, 0x00, 0x33, 0x00, 0x34, 0x00,
	0x2d, 0x00, 0x31, 0x00, 0x32, 0x00, 0x33, 0x00, 0x34, 0x00, 0x2d, 0x00, 0x31, 0x00, 0x32, 0x00, 0x33, 0x00,
	0x34, 0x00, 0x35, 0x00, 0x36, 0x00, 0x37, 0x00, 0x38, 0x00, 0x39, 0x00, 0x41, 0x00, 0x42, 0x00, 0x43, 0x00,
	0x7d, 0x00, 0x00, 0x00
};

const UINT8 CompactId[] =
{
	0x28, 0x00, 0x00, 0x00, 0x00, 0x01, 0x04, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
	0x57, 0x49, 0x4e, 0x55, 0x53, 0x42, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

UINT8 GetStatus[] =
{
	0x01, 0x00
};
