/********************************** (C) COPYRIGHT *******************************
* File Name          : uvc.c
* Author             : WCH
* Version            : V1.0
* Date               : 2020/12/31
* Description        : usb3.0 Library function
* Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
*******************************************************************************/

#include "UVCLIB.H"

#define NoError         0x00
#define NotReady        0x01
#define WrongState      0x02
#define Power           0x03
#define OutOfRange      0x04
#define InvalidUnit     0x05
#define InvalidControl  0x06
#define InvalidRequest  0x07
#define InvalidValueWithinRange 0x08
#define Unknown         0xFF
/*Channel information is the configuration value*/
//wValue,According to the agreement,wIndex indicates  Entity ID
/*info
 * D0:1=Supports GET value requests                                 Capability
 * D1:1=Supports SET value requests                                 Capability
 * D2:1=Disabled due to automatic mode (under device control)       State
 * D3:1= Autoupdate Control                                         Capability
 * D4:1= Asynchronous Control                                       Capability
 * D5:1= Disabled due to incompatibility with Commit state.         State
 * D6-7:Reserved (Set to 0)
 * */

typedef struct
{
    UINT8 info[1];
    UINT8 len[2];   //Used by the expansion unit
    UINT8 min[10];
    UINT8 max[10];
    UINT8 res[10];
    UINT8 def[10];
    UINT8 cur[10];
    UINT8 set[10];
}Unit_t;

typedef struct __attribute__((packed))
 {
    UINT8 len;
    UINT8 tog;
    UINT32 Frequeny;
    UINT32 Clock;
    UINT16 count;
    UINT32 reallen;
 }Picture;

 Picture picture;

 volatile UINT32 addr_cnt;
 volatile UINT32 start1;
 volatile UINT32 end1;
 volatile UINT32 fcnt;
 volatile UINT32V class=0;
 UINT32V UVC_DVP;

UINT8V get_cur[]={
        0x00, 0x00,
        0x01,
        0x01,
        0x15,0x16,0x05,0x00,
        0x00, 0x00,
        0x00, 0x00,
        0x00, 0x00,
        0x00, 0x00,
        0x00, 0x00,
        0x00, 0x48, 0x3f, 0x00,
        0x00, 0x00, 0x00, 0x00
};

Unit_t VAICSID[]={//ID=0,send VideoControl Interface Control Selectors
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//VC _CONTROL_ UNDEFINED
        {
                0,{0x01,0x00},
                {0},{0},{0},{0},{0x10},{0}
        },//VC_VIDEO_POWER_MODE_CONTROL
        {
                0,{0x01,0x00},
                {0},{0},{0},{0},{NoError},{0}
        },//VC_REQUEST_ERROR_CODE_CONTROL
        {}//Reserved


};
Unit_t CameraID[]={//ID 01
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//CT_CONTROL_UNDEFINED 0
        {
                0x03,{0x01,0x00},
                {0},{0},{0},{0},{0},{0}

        },//CT_SCANNING_MODE_CONTROL 1
        {
                0x03,{0x01,0x00},
                {0},{0},{0},{0},{0x06},{0}
        },//CT_AE_MODE_CONTROL 2
        {
                0x3,{0x01,0x00},
                {0x00},{0x01},{0x01},{0x00},{0},{0}
        },//CT_AE_PRIORITY_CONTROL 3
        {
                0x0f,{0x04,0x00},
                {0x32,0x00,0x00,0x00},{0x10,0x27,0x00,0x00},{0x01,0x00,0x00,0x00},{0xa6,0x00,0x00,0x00},{0},{0}
        },//CT_EXPOSURE_TIME_ABSOLUTE_CONTROL 4
        {
                0x00,{0x00,0x00},
                {0},{0},{0},{0},{0},{0}
        },//CT_EXPOSURE_TIME_RELATIVE_CONTROL 5
        {
                0x0f,{0x02,0x00},
                {0x00,0x00},{0xff,0x03},{0x01,0x00},{0x44,0x00},{0x44,0x00},{0}
        },//CT_FOCUS_ABSOLUTE_CONTROL 6
        {
                0x00,{0x00,0x00},
                {0},{0},{0},{0},{0},{0}
        },//CT_FOCUS_RELATIVE_CONTROL 7
        {
                0,{0x01},
                {0},{0},{0},{0},{0x01},{0}
        },//CT_FOCUS_AUTO_CONTROL 8
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//CT_IRIS_ABSOLUTE_CONTROL 9
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//CT_IRIS_RELATIVE_CONTROL a
        {
                0x03,{0x02,0x00},
                {0x00,0x00},{0x03,0x00},{0x01,0x00},{0x00,0x00},{0},{0}
        },//CT_ZOOM_ABSOLUTE_CONTROL b
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//CT_ZOOM_RELATIVE_CONTROL c
        {
                0x03,{0x08,0x00},
                {0x00, 0x1f, 0xff, 0xff,  0x40, 0x57, 0xff, 0xff },
                {0x00, 0xe1, 0x00, 0x00,  0xc0, 0xa8, 0x00, 0x00},
                {0x10, 0x0e, 0x00, 0x00,  0x10, 0x0e, 0x00, 0x00},{0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00},{0},{0}
        },//CT_PANTILT_ABSOLUTE_CONTROL d
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//CT_PANTILT_RELATIVE_CONTROL e
        {
                0x03,{0x02,0x00},
                {0},{0x03,0x00},{0x01,0x00},{0},{0},{0}
        },//CT_ROLL_ABSOLUTE_CONTROL f
        {
                0x03,{0x02,0x00},
                {0x00, 0x00},{0x03, 0x00},{0x01, 0x00},{0x00, 0x00},{0},{0}
        },//CT_ROLL_RELATIVE_CONTROL
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//CT_PRIVACY_CONTROL
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//CT_FOCUS_SIMPLE_CONTROL
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//CT_WINDOW_CONTROL
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        }//CT_REGION_OF_INTEREST_CONTROL
};

Unit_t ProcessID[]={//ID 02
        {
                0,{0x00,0x00},
                {0},{0},{0},{0},{0},{0}
        },//PU_CONTROL_UNDEFINED 0
        {
                0x03,{0x02,0x00},
                {0x00, 0x00},{0x02, 0x00},{0x01, 0x00},{0x00, 0x00},{0},{0}
        },//PU_BACKLIGHT_COMPENSATION_CONTROL 1
        {
                0x03,{0x02,0x00},
                {0xc0, 0xff},{0x40, 0x00 },{0x01, 0x00},{0x00, 0x00},{0},{0}
        },//PU_BRIGHTNESS_CONTROL 2
        {
                0x03,{0x02,0x00},
                {0x00, 0x00},{0x64, 0x00},{0x01, 0x00 },{0x32, 0x00},{0},{0}
        },//PU_CONTRAST_CONTROL 3
        {
                0,{0x00,0x00},
                {0},{0},{0},{0},{0},{0}
        },//PU_GAIN_CONTROL 4
        {
                0x03,{0x01,0x00},
                {0x00},{0x02},{0x01},{0x01},{0x01},{0}
        },//PU_POWER_LINE_FREQUENCY_CONTROL 5
        {
                0x03,{0x02,0x00},
                {0x4c, 0xff},{ 0xb4, 0x00},{0x01, 0x00 },{0x00, 0x00},{0},{0}
        },//PU_HUE_CONTROL 6
        {
                0x03,{0x02,0x00},
                {0x00, 0x00},{0x64, 0x00},{0x01, 0x00},{0x40, 0x00},{0},{0}
        },//PU_SATURATION_CONTROL 7
        {
                0x03,{0x02,0x00},
                {0x00, 0x00},{0x64, 0x00},{0x01, 0x00},{0x32, 0x00},{0},{0}
        },//PU_SHARPNESS_CONTROL 8
        {
                0x03,{0},
                {0x64, 0x00},{0xf4, 0x01},{0x01, 0x00},{0x2c, 0x01},{0},{0}
        },//PU_GAMMA_CONTROL 9
        {
                0x0f,{0x02,0x00},
               {0xf0, 0x0a},{0x64, 0x19},{0x0a, 0x00},{0xf8, 0x11},{0},{0}
        },//PU_WHITE_BALANCE_TEMPERATURE_CONTROL a
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//PU_WHITE_BALANCE_TEMPERATURE_AUTO_CONTROL
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//PU_WHITE_BALANCE_COMPONENT_CONTROL
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//PU_WHITE_BALANCE_COMPONENT_AUTO_CONTROL
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//PU_DIGITAL_MULTIPLIER_CONTROL
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//PU_DIGITAL_MULTIPLIER_LIMIT_CONTROL
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//PU_HUE_AUTO_CONTROL
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//PU_ANALOG_VIDEO_STANDARD_CONTROL
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        },//PU_ANALOG_LOCK_STATUS_CONTROL
        {
                0,{0},
                {0},{0},{0},{0},{0},{0}
        }//PU_CONTRAST_AUTO_CONTROL

};
Unit_t ExtensionID1[]={//ID 04
        {0},{0},{0},{0},{0},{0},{0},{0},{0},{0},
        {
                0x03,{0x08,0x00},
                {0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00},
                {0xff, 0xff, 0xff, 0xff,  0xff, 0xff, 0xff, 0xff },
                {0x01, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00},{0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00},{0},{0}
        },//a
        {
                0x03,{0x08,0x00},
                {0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00},
                {0xff, 0xff, 0xff, 0xff,  0xff, 0xff, 0xff, 0xff },
                {0x01, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00},{0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00},{0},{0}
        }//b
};
Unit_t ExtensionID2[]={//ID 06
        {0},
        {
                0x03,{0x02,0x00},
                {0x00, 0x00},{0xff, 0xff},{0x00, 0x00},{0x01, 0x00},{0},{0}
        },
        {
                0x03,{0x02,0x00},
                {0xfa, 0xff},{0x06, 0x00},{0x40, 0x00},{0x00, 0x00},{0},{0}
        },
        {
                0x01,{0x02,0x00},
                {0x00, 0x00},{0x00, 0x00},{0x00, 0x00},{0x00, 0x00},{0},{0}
        },
        {
                0x03,{0x01,0x00},
                {0x00, 0x00},{0x00, 0x00},{0x00, 0x00},{0x00, 0x00},{0},{0}
        },
        {
                0x03,{0x01,0x00},
               {0x00, 0x00},{0x00, 0x00},{0x00, 0x00},{0x00, 0x00},{0},{0}
        },
        {
                0x03,{0x01,0x00},
                {0x00, 0x00},{0x00, 0x00},{0x00, 0x00},{0x00, 0x00},{0},{0}
        },{0},{0},{0},{0},{0},
        {0},{0},{0},{0},{0},{0},{0},
        {
                0x03,{0x01,0x00},
               {0x00, 0x00},{0x01, 0x00},{0x01, 0x00},{0x00, 0x00},{0},{0}
        },
        {
                0x03,{0x0a,0x00},
                {0},{0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00,  0x09, 0x00 },{0},{0},{0},{0}
        },
        {
                0x01,{0x04,0x00},
                {0},{0},{0},{0},{0},{0}
        },
        {
                0x03,{0x01,0x00},
                {0x00, 0x00},{0x01, 0x00},{0x01, 0x00},{0x00, 0x00},{0},{0}
        },
        {
                0x03,{0x01,0x00},
               {0x01, 0x00},{0x03, 0x00},{0x02, 0x00},{0x01, 0x00},{0},{0}
        },
        {
                0x03,{0x01,0x00},
                {0x01, 0x00},{0x00, 0x00},{0x01, 0x00},{0x01, 0x00},{0},{0}
        }
};
/*******************************************************************************
 * @fn      Res_NonStandardReq
 *
 * @brief   Nonstandard request processing function
 *
 * @return  Length
 */
UINT16 Res_NonStandardReq(UINT8 **pDescr){
    if(UsbSetupBuf->bRequestType==0xA1){
        switch(UsbSetupBuf->wIndex.bw.bb0){//UsbSetupBuf->wValue.bw.bb0
                    case 00:
                        switch(UsbSetupBuf->wIndex.bw.bb1){
                            case 00://Only send request to the VideoControl interface (interface ID 00)
                                switch(UsbSetupBuf->bRequest){
                                    case 0x81://GET CUR
                                                *pDescr = (PUINT8)&VAICSID[UsbSetupBuf->wValue.bw.bb0].cur;
                                                break;
                                }
                                break;
                            case 01://
                                switch(UsbSetupBuf->bRequest){
                                        case 0x81://GET CUR
                                        case 0x82:
                                        case 0x83:
                                                *pDescr = (PUINT8)&get_cur;
                                                break;
                                }
                                break;
                            }
                        break;
                    case 01:
                        switch(UsbSetupBuf->bRequest){//LEN->INFO->MIN->MAX->RES->DEF
                            case 0x81://GET CUR
                                *pDescr = (PUINT8)&CameraID[UsbSetupBuf->wValue.bw.bb0].cur;
                                break;
                            case 0x82://GET MIN
                                *pDescr = (PUINT8)&CameraID[UsbSetupBuf->wValue.bw.bb0].min;
                                break;
                            case 0x83://GET MAX
                                *pDescr = (PUINT8)&CameraID[UsbSetupBuf->wValue.bw.bb0].max;
                                break;
                            case 0x84://GET RES
                                *pDescr = (PUINT8)&CameraID[UsbSetupBuf->wValue.bw.bb0].res;
                                break;
                            case 0x85://GET LEN
                                *pDescr = (PUINT8)&CameraID[UsbSetupBuf->wValue.bw.bb0].len;
                                break;
                            case 0x86:// GET INFO
                                *pDescr = (PUINT8)&CameraID[UsbSetupBuf->wValue.bw.bb0].info;
                                break;
                            case 0x87://GET DEF
                                *pDescr = (PUINT8)&CameraID[UsbSetupBuf->wValue.bw.bb0].def;
                                break;
                        }
                        break;
                    case 02:
                        switch(UsbSetupBuf->bRequest){//LEN->INFO->MIN->MAX->RES->DEF
                            case 0x81://GET CUR
                                *pDescr = (PUINT8)&ProcessID[UsbSetupBuf->wValue.bw.bb0].cur;
                                break;
                            case 0x82://GET MIN
                                *pDescr = (PUINT8)&ProcessID[UsbSetupBuf->wValue.bw.bb0].min;
                                break;
                            case 0x83://GET MAX
                                *pDescr = (PUINT8)&ProcessID[UsbSetupBuf->wValue.bw.bb0].max;
                                break;
                            case 0x84://GET RES
                                *pDescr = (PUINT8)&ProcessID[UsbSetupBuf->wValue.bw.bb0].res;
                                break;
                            case 0x85://GET LEN
                                *pDescr = (PUINT8)&ProcessID[UsbSetupBuf->wValue.bw.bb0].len;
                                break;
                            case 0x86:// GET INFO
                                *pDescr = (PUINT8)&ProcessID[UsbSetupBuf->wValue.bw.bb0].info;
                                break;
                            case 0x87://GET DEF
                                *pDescr = (PUINT8)&ProcessID[UsbSetupBuf->wValue.bw.bb0].def;
                                break;
                        }
                        break;
                    case 04:
                        switch(UsbSetupBuf->bRequest){//LEN->INFO->MIN->MAX->RES->DEF
                            case 0x81://GET CUR
                                *pDescr = (PUINT8)&ExtensionID1[UsbSetupBuf->wValue.bw.bb0].cur;
                                break;
                            case 0x82://GET MIN
                                *pDescr = (PUINT8)&ExtensionID1[UsbSetupBuf->wValue.bw.bb0].min;
                                break;
                            case 0x83://GET MAX
                                *pDescr = (PUINT8)&ExtensionID1[UsbSetupBuf->wValue.bw.bb0].max;
                                break;
                            case 0x84://GET RES
                                *pDescr = (PUINT8)&ExtensionID1[UsbSetupBuf->wValue.bw.bb0].res;
                                break;
                            case 0x85://GET LEN
                                *pDescr = (PUINT8)&ExtensionID1[UsbSetupBuf->wValue.bw.bb0].len;
                                break;
                            case 0x86:// GET INFO
                                *pDescr = (PUINT8)&ExtensionID1[UsbSetupBuf->wValue.bw.bb0].info;
                                break;
                            case 0x87://GET DEF
                                *pDescr = (PUINT8)&ExtensionID1[UsbSetupBuf->wValue.bw.bb0].def;
                                break;
                        }
                        break;
                    case 06:
                        switch(UsbSetupBuf->bRequest){//LEN->INFO->MIN->MAX->RES->DEF
                            case 0x81://GET CUR
                                *pDescr = (PUINT8)&ExtensionID2[UsbSetupBuf->wValue.bw.bb0].cur;
                                break;
                            case 0x82://GET MIN
                                *pDescr = (PUINT8)&ExtensionID2[UsbSetupBuf->wValue.bw.bb0].min;
                                break;
                            case 0x83://GET MAX
                                *pDescr = (PUINT8)&ExtensionID2[UsbSetupBuf->wValue.bw.bb0].max;
                                break;
                            case 0x84://GET RES
                                *pDescr = (PUINT8)&ExtensionID2[UsbSetupBuf->wValue.bw.bb0].res;
                                break;
                            case 0x85://GET LEN
                                *pDescr = (PUINT8)&ExtensionID2[UsbSetupBuf->wValue.bw.bb0].len;
                                break;
                            case 0x86:// GET INFO
                                *pDescr = (PUINT8)&ExtensionID2[UsbSetupBuf->wValue.bw.bb0].info;
                                break;
                            case 0x87://GET DEF
                                *pDescr = (PUINT8)&ExtensionID2[UsbSetupBuf->wValue.bw.bb0].def;
                                break;
                        }
                        break;
                    }

        if(UsbSetupBuf->wIndex.bw.bb0 ==0x01 &&((UsbSetupBuf->wValue.bw.bb0==0x14) ||(UsbSetupBuf->wValue.bw.bb0==0x09)))
        {
                VAICSID[2].cur[0]=(UINT8)InvalidControl;
                return 0xFFFF;
        }
    }
    return 0;
}

/*******************************************************************************
 * @fn      clearError
 *
 * @brief   clear error
 *
 * @return  None
 */
void clearError(){
    VAICSID[UsbSetupBuf->wValue.bw.bb0].cur[0]=NoError;
}

/*******************************************************************************
 * @fn      ctrlCamera
 *
 * @brief   USB3.0 set interface processing function
 *
 * @return  None
 */
void ctrlCamera(){
    class = 1;
    if(UsbSetupBuf->wValue.bw.bb1 > 0){//on
        picture.Clock = 0x0136E5;
        picture.Frequeny=0;
        picture.count=0x04DD;
        picture.len=0x0c;
        picture.tog = 0x8c;
        picture.reallen=0x0C;
        UVC_DVP = 0x01;
        memcpy(endp1RTbuff,  (uint8_t *)&picture,12 );
        USB30_setTxCtrl(endp_1,ENABLE,ACK_TP,NUMP_1,picture.reallen);
    }else{//off
        UVC_DVP = 0x00;
    }
}

/*******************************************************************************
 * @fn      ctrlCamera_hs
 *
 * @brief   USB2.0 set interface processing function
 *
 * @return  None
 */
void ctrlCamera_hs(){
    class = 1;
    if( UsbSetupBuf->wValue.bw.bb1 > 0){//on
        picture.Clock = 0x0136E5;
        picture.Frequeny=0;
        picture.count=0x04DD;
        picture.len=0x0c;
        picture.tog = 0x8c;
        picture.reallen=0x0C;
        UVC_DVP = 0x01;
        memcpy(endp1RTbuff,  (uint8_t *)&picture,12 );
        R16_UEP1_T_LEN =  12;
    }else{//off
        UVC_DVP = 0x00;
    }
}

/*******************************************************************************
 * @fn      endp1_Hander
 *
 * @brief   USB3.0 endpoint 1 processing function
 *
 * @return  None
 */
void endp1_Hander(void){
                class++;
                picture.Clock+=0x753;
                if(class%8==0){
                //Write structure every 8, add one for each IPT , 125us
                picture.count =(++picture.count)&0x07ff;
                }
                if(fcnt==1){//Frame data
                        picture.reallen = 524;
                        fcnt=2;
                }
                if(end1==1){//Frame END
                        picture.tog |= 0x02;
                        end1=0;//tog end set 1
                        fcnt=0;
                }
                memcpy( endp1RTbuff,  (uint8_t *)&picture,12 );
                USB30_setTxCtrl(endp_1,ENABLE,ACK_TP,NUMP_1,picture.reallen);
                picture.reallen = 12;
                picture.tog &= 0xFD;

}

/*******************************************************************************
 * @fn      endp1_Hander_hs
 *
 * @brief   USB2.0 endpoint 1 processing function
 *
 * @return  None
 */
void endp1_Hander_hs(void){

        class++;
        picture.Clock+=0x753;
        if(class%8==0){
        //Write structure every 8, add one for each IPT , 125us
        picture.count =(++picture.count)&0x07ff;
        }
        if(fcnt==1){//Frame data
                picture.reallen = 524;
                fcnt=2;
        }
        if(end1==1){//Frame END
                picture.tog |= 0x02;
                end1=0;//tog end set 1
                fcnt=0;
        }
        memcpy( endp1RTbuff,  (uint8_t *)&picture,12 );

        R16_UEP1_T_LEN =  picture.reallen;

        picture.reallen = 12;
        picture.tog &= 0xFD;

}

/*******************************************************************************
 * @fn      DVP_Hander
 *
 * @brief   DVP Interrupt processing function
 *
 * @return  None
 */
void DVP_Hander(void){
    UINT32 j;
    if (R8_DVP_INT_FLAG & RB_DVP_IF_ROW_DONE) //End of line interrupt
    {
        R8_DVP_INT_FLAG = RB_DVP_IF_ROW_DONE;
        if (addr_cnt%2)     //buf1 done
        {
            memcpy((UINT8 *)(endp1RTbuff+12),(UINT8 *)JPEG_DVPDMAaddr1,512);
        }
        else
        {
            memcpy((UINT8 *)(endp1RTbuff+12),(UINT8 *)JPEG_DVPDMAaddr0,512);
        }
        addr_cnt++;
        fcnt = 1;
    }
    else if (R8_DVP_INT_FLAG & RB_DVP_IF_FRM_DONE) //Frame reception completion interrupt
    {
        R8_DVP_INT_FLAG = RB_DVP_IF_FRM_DONE;
    }

    else if (R8_DVP_INT_FLAG & RB_DVP_IF_STR_FRM) //Frame start interrupt
    {
        R8_DVP_INT_FLAG = RB_DVP_IF_STR_FRM;
        start1 =1;
        if(start1&&(UVC_DVP & 0x01)==1){
                picture.tog ^=0x01;//Frame start
                picture.Frequeny +=0x7A0A3;
                start1 = 0;
                addr_cnt = 0;
        }
    }

    else if (R8_DVP_INT_FLAG & RB_DVP_IF_STP_FRM) //Frame end interrupt
    {
        R8_DVP_INT_FLAG = RB_DVP_IF_STP_FRM;
        end1=1;
    }

    else if (R8_DVP_INT_FLAG & RB_DVP_IF_FIFO_OV) //FIFO Overflow interrupt
    {
        R8_DVP_INT_FLAG = RB_DVP_IF_FIFO_OV;
        printf("FIFO OV\r\n");
    }
}
